<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:render="http://software.in2p3.fr/lavoisier/rendering.xsd">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/><!-- prevents text() function to ignore the text nodes other than 1st one -->
    <xsl:param name="view">Default rendering</xsl:param>

    <xsl:template match="/render:rendering" priority="1"><render:rendering title="{$view}"><xsl:apply-templates/></render:rendering></xsl:template>
    <xsl:template match="render:fields" priority="1"><render:fields><xsl:apply-templates/></render:fields></xsl:template>
    <xsl:template match="render:field" priority="1"><render:field label="{@label}"><xsl:apply-templates/></render:field></xsl:template>
    <xsl:template match="render:table" priority="1"><render:table name="{@name}"><xsl:apply-templates/></render:table></xsl:template>
    <xsl:template match="render:column_labels" priority="1"><render:column_labels><xsl:apply-templates/></render:column_labels></xsl:template>
    <xsl:template match="render:column_label" priority="1"><render:column_label><xsl:apply-templates/></render:column_label></xsl:template>
    <xsl:template match="render:row" priority="1"><render:row><xsl:apply-templates/></render:row></xsl:template>
    <xsl:template match="render:column[not(@link)]" priority="1"><render:column><xsl:apply-templates/></render:column></xsl:template>
    <xsl:template match="render:column[@link]" priority="1"><render:column link="{@link}"><xsl:apply-templates/></render:column></xsl:template>

    <xsl:template match="/*">
        <render:rendering title="{$view}">
            <xsl:apply-templates select="." mode="ROOT"/>
        </render:rendering>
    </xsl:template>
    <xsl:template match="*[count(*) = 1]" mode="ROOT">
        <xsl:apply-templates mode="ROOT"/>
    </xsl:template>
    <xsl:template match="*" mode="ROOT">
        <xsl:variable name="single" select="*[not(local-name()=local-name(preceding-sibling::*[1]) or local-name()=local-name(following-sibling::*[1]))]"/>
        <xsl:variable name="multiple"   select="*[local-name()=local-name(preceding-sibling::*[1]) or local-name()=local-name(following-sibling::*[1])]"/>
        <render:fields>
            <xsl:for-each select="@* | $single[not(*)][normalize-space(text())!='']">
                <render:field label="{local-name()}">
                    <xsl:value-of select="."/>
                </render:field>
            </xsl:for-each>
        </render:fields>
        <xsl:choose>
            <xsl:when test="$multiple">
                <xsl:apply-templates mode="TABLE"
                                     select="$multiple[not(local-name()=local-name(preceding-sibling::*[1]))]"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates mode="TABLE"
                                     select="$single/*[not(local-name()=local-name(preceding-sibling::*[1]))]"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="*" mode="TABLE">
        <render:table>
            <xsl:choose>
                <xsl:when test="following-sibling::*[local-name()=local-name(current())]">
                    <!-- current element is first row -->
                    <xsl:attribute name="name"><xsl:value-of select="local-name()"/></xsl:attribute>
                    <xsl:call-template name="TABLE">
                        <xsl:with-param name="rows" select=". | following-sibling::*[local-name()=local-name(current())]"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <!-- current element is parent of rows -->
                    <xsl:attribute name="name">
                        <xsl:choose>
                            <xsl:when test="@id|@ID"><xsl:value-of select="@id|@ID"/></xsl:when>
                            <xsl:when test="@name|@NAME"><xsl:value-of select="@name|@NAME"/></xsl:when>
                            <xsl:when test="count(@*)=1"><xsl:value-of select="@*"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="local-name()"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                    <xsl:call-template name="TABLE">
                        <xsl:with-param name="rows" select="*"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </render:table>
    </xsl:template>
    <xsl:template name="TABLE">
        <xsl:param name="rows"/>
        <xsl:apply-templates select="$rows[1]" mode="HEADER"/>
        <xsl:for-each select="$rows">
            <xsl:apply-templates select="$rows[1]" mode="ROW">
                <xsl:with-param name="current" select="."/>
            </xsl:apply-templates>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="@* | *" mode="LABEL">
        <render:column_label>
            <xsl:if test="number(.)=.">
                <xsl:attribute name="unit">value</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="local-name()"/>
        </render:column_label>
    </xsl:template>
    <xsl:template match="*" mode="HEADER">
        <render:column_labels>
            <xsl:apply-templates select="@*" mode="LABEL"/>
            <xsl:for-each select="*">
                <xsl:if test="not((preceding::*|following-sibling::*)[local-name()=local-name(current())])">
                    <xsl:apply-templates select="." mode="LABEL"/>
                </xsl:if>
            </xsl:for-each>
        </render:column_labels>
    </xsl:template>
    <xsl:template match="*" mode="ROW">
        <xsl:param name="current"/>
        <render:row>
            <xsl:for-each select="@*">
                <xsl:variable name="value" select="$current/@*[local-name()=local-name(current())]"/>
                <render:column><xsl:value-of select="$value"/></render:column>
            </xsl:for-each>
            <xsl:for-each select="*">
                <xsl:if test="not((preceding::*|following-sibling::*)[local-name()=local-name(current())])">
                    <xsl:variable name="value" select="$current/*[local-name()=local-name(current())]"/>
                    <render:column><xsl:value-of select="$value"/></render:column>
                </xsl:if>
            </xsl:for-each>
        </render:row>
    </xsl:template>
</xsl:stylesheet>